// -------------------------------------------------------------------------------------------------------------
// ------------                 POCSAG Decoder mit Handy Interface                  ----------------------------
// -------------------------------------------------------------------------------------------------------------
// --------        Version 5 - November 2012               Autor:  Dane                     --------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// Controller: Atmel AtMega 324 P
// externer Quarz: 18,432 MHz
// 2 interne UART: 0 = Handy / 1 = PC
// -------------------------------------------------------------------------------------------------------------
// Diese Version wickelt Dekodierung zum Groteil in der Interruptroutine ab
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// Baudrate der bertragung bei 1200. Ermittlung der genauen Baudrate erfolgt whrend der Prambel
// -------------------------------------------------------------------------------------------------------------

#define F_CPU 18432000UL
#define BAUDRATE_UART0 19200UL
#define BAUDRATE_UART1 19200UL
#define uart_delay 4                        //delay in ms zwischen UART bertragungen

#include <avr/io.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <string.h>
#include <stdio.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>

#include "uart_dual.c"                      // Lib fr die beiden UARTs (ohne Interrupts)
#include "own_handy.c"                      // Lib fr die Ansteuerung des Handys (Siemens M35)


// Funktionsprototypen
void waitNachAlarm(void);                   // Prototyp Funktion zur "Reload-Sperre"
char *getStichwortText(char * stichwort);   // Hinweistext fr Stichwortkrzel ausgeben
void RIC1_Alarm(void);                       // Funktion bei erkannter RIC1-Nachricht
void wait_preamble(int timerwert);          // Auf Prambelende warten
int baudanpassung(void);                    // Baudrate erkennen
void alarmwahl(void);                       // Funktion zum weiteren Vorgehen bei erkannter Alarmierung

// Boolsche Variable deklarieren
#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

//Globale Variablen
volatile unsigned long RIC = 0;             // Adresse der alarmierten "Schleife"
unsigned short int Funktion = 0;            // Zahlenwert der Funktionsbits 
char Funktion_text[9] = "        ";         // Klartext der Funktion
volatile int timeoutFlag_preamble = 0;      // Fuer Timeout bei Preambelerkennung, durch Timer2
unsigned char meldung[60]= "                                            ";  //Klartext der Meldung
char linefeed[] = {10,0};                   // String fr Zeilenumbruch

// Konstanten / Einstellungen
const int minPreamble = 40;                 //mindestanzahl wechsel bei preamble
const int maxPreamble = 600;                //maxZahl Bits bei Preambleerkennung
const int counterwert = 15360;              //Wert des Timers bei Baudrate von 1200 bei verwendetem Quarz und Timereinstellung (= Taktrate[Hz] / Baudrate)
const int abweichung = 50;                  //max +- Abweichung des BaudratenTimerWertes (counterwert)

const short int SCW[32] =       {0,1,1,1,1,1,0,0,1,1,0,1,0,0,1,0,0,0,0,1,0,1,0,1,1,1,0,1,1,0,0,0};  // Synchronwort
const short int idlew[32] = 	{0,1,1,1,1,0,1,0,1,0,0,0,1,0,0,1,1,1,0,0,0,0,0,1,1,0,0,1,0,1,1,1};  // Idle-Wort (leeres Codewort)

// Stringkonstanten fr Stichwortkrzel (werden in FLASH Speicher ausgelagert, da SRAM bereits voll)
	const char A1[] PROGMEM = "\"Anfrage 1\"";
	const char A2[] PROGMEM = "\"Anfrage 2\"";
	const char A3[] PROGMEM = "\"News\"";
	const char I1[] PROGMEM = "\"Info 1\"";
	const char I2[] PROGMEM = "\"Info 2\"";
	const char I3[] PROGMEM = "\"Wetter\"";


// Alarmschleifen  Adressen
const unsigned long RIC_1				= 100000;		// Adresse 1
const unsigned long RIC_2               = 200000;       // Adresse 2
const unsigned long RIC_3               = 300000;       // Adresse 3



// Variablen in Interruptroutine
volatile short int bitwert = 0;                         // der aktuelle Bitwert (0 oder 1) am Empfnger
volatile int trigger = FALSE;                           // ist FALSE und wird auf TRUE gesetzt, wenn Timer ausgelst hat fr nchstes Bit
volatile uint16_t bitnr = 0;                            // temp. Zhler fr Bits

volatile unsigned short int status = 0;                 // bei interrupt modus: zeigt status der aktion an
unsigned short int zeilennr = 0;                        // Nr des Codeworts (0-16)
unsigned int parity = 0;                                // Parittszhler
unsigned char meldungsbatch[300] = {0};                 // Sammelarray fr Bits der Meldung-Codewrter
unsigned short int meldungsbatchcounter = 0;            // Zhler fr Anzahl an Batches in denen Meldung aufgeteilt
unsigned short int batchnr = 0;                         // Batch-Zhler
unsigned long multi = 1048576;                          // multiplikator fr RIC berechnung
unsigned long RIC_array[30]={0};                        // Array fr die erkannten Adressen (debug-zwecke)
volatile unsigned int RIC_cntr = 0;                     // Anzahl erkannter Adressen (die dann im Array stehen - debugzwecke)

volatile unsigned short int signals = 0;                // Damit wird der Main-Funktion die Art der Erkennung mitgeteilt




// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------- Interrupt Routine des POCSAG-Baudraten-Timers ------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
//   Diese wird von Timer1 ausgelst, immer in der Mitte eines Bits
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

ISR (TIMER1_COMPA_vect)
{
	bitwert = (PINB & 1);           //aktueller Bitwert wird erfasst
    bitnr++;
    
    // State-Machine
    // Werte der Variablen "state" im Anhang
    switch(status){
        // Preamble abwarten - wird auserhalb der Interruptroutine gemacht, deshalb nur auf den Trigger setzen
        case 0:     
            trigger = TRUE;                 // Trigger gesetzt
            signals = 0;
			zeilennr = 0;
            break;
            
        // abgleich auf SCW
        case 1:
            if(bitnr==32){
				zeilennr=0;
				if(bitwert!=(SCW[(bitnr-1)])){
                    status=90;  //SCW Fehler
                    break;
				}else{				// Ende SCW und korrekt
					zeilennr++;
					status = 2;	
					bitnr = 0;
					wdt_reset();
				}			
			 }else if(bitwert!=(SCW[(bitnr-1)])){
                    status=90;  //SCW Fehler
                    break;
             }
            
            break;
        
        // adresszeile warten (erstes bit == 0)
        case 2:		
            if((bitnr!=1)&&(bitnr<32)){
				break;
            }else if(bitnr==32){  // zeile zuende
                bitnr=0;
                if(zeilennr<16){
                    zeilennr++;
					break;
                }else{              // Batch zuende, ggf. weiteres scw prfen
                    zeilennr=0;
					batchnr++;
                    status = 1;
                    break;
                }
            }else if((bitnr==1)&&(bitwert!=0)){     //1.bit != 0 , d.h. keine Adresse
				break;
            }else if((bitnr==1)&&(bitwert==0)){     //1.bit = 0, adresse folgt!
				Funktion=0;
                status=3;
                multi = 1048576;
				RIC=0;
				parity=0;
				break;
            }
            
            break;
        
        // adresse wandeln
        case 3:
            if(bitnr<20){
                RIC = RIC + (bitwert * multi);
				multi = multi/2;
                parity = parity + bitwert;
            }else{
				parity = parity + bitwert;
                if(bitnr<32){
                    if((bitnr==20)||(bitnr==21)){
                        Funktion = Funktion + (bitwert*(22-bitnr));
                    }
                }else{	// zeile ende - bzw. aufruf bei letztem bit dieser zeile
					RIC = RIC + (int)((zeilennr-1)/2); 
                    multi = 1048576;    // Startwert wiederherstellen
                    bitnr = 0;
                    
                    if((parity%2)!=0){  // Parity-Error
						if(zeilennr<16){
							zeilennr++;
							status = 2;
						}else{
							zeilennr=0;
							status=1;
						}												
                        RIC_array[RIC_cntr]=8;    // RIC in Array speichern
                        RIC_cntr++;
						parity=0;
						multi = 1048576;
                        break;
                    }else{              // Parity OK
						wdt_reset();
						if((RIC<260000)&&(RIC>250000)){
							RIC_array[RIC_cntr]=RIC;    // RIC in Array speichern
							RIC_cntr++;
						}						
						parity=0;
                        if((RIC!=RIC_1)&&(RIC!=RIC_2)&&(RIC!=RIC_3)){
                            if(zeilennr<16){
								zeilennr++;
								status = 2;     // RIC nicht aktiv, nchste RIC suchen
							}else{
								zeilennr=0;
								status = 1;
							}														
                        }else{
							// RIC bekannt
                            status=5;
						}
							}                    
					}
            }
            break;
            
    
        
        // RIC aktiv
        case 5:
			bitnr = 1;
			meldungsbatchcounter=0;
            if(zeilennr<16){   
				zeilennr++;
                if(bitwert!=1){ // keine Meldung folgt der Adresse
                    status = 60;    // TONRUF
                }else{
                    status = 6;     // Meldung folgt
                    break;
                }
            }else{
				// batch zuende, scw des folgenden batch pruefen und dann meldung kopieren
                status = 50;    // scw weiterer batch pruefen
                batchnr++;
                zeilennr = 0;
                break;
            }
            break;
            
            
        //MELDUNG UMKOPIEREN
        case 6:
            if((bitnr==1)&&(bitwert!=1)){       
                status=7;                      // Meldung zuende
                break;
			}else if((bitnr==1)&&(bitwert==1)){
				break;			
            }else if(bitnr<22){
                meldungsbatch[meldungsbatchcounter] = bitwert;      // Array mit message bits an einem stueck
                meldungsbatchcounter++;                             // schreibzaehler auf bit im array
				break;
            }else{	// auf Zeilenende warten
                if(bitnr==32){
                    if(zeilennr<16){
						zeilennr++;
						bitnr=0;
					}else{
						status = 55;	// Batch zuende, naechstes SCW pruefen 
						batchnr++;
						zeilennr=0;
						bitnr=0;
					}					
                }       
            }
           
            break;
            
            
            // Meldung in Klartext umwandeln
        case 7:
			wdt_reset();
            if(meldungsbatchcounter<7){         // Meldungsbitstrom zu kurz, dann leeren und Tonruf machen
                int k = 0;
                for(k=0;k<300;k++){         
                    meldungsbatch[k]=0;         // Array leeren
                }
                status = 60;                    // Tonruf
                break;
            }else{
                // Umwandlung des Bitstroms in Zeichen (7bit ASCII)
                // vorgehen: 7 bit abschneiden und rueckwaerts umwandeln in Char
                // Meldungsstring mit \0 abschlieen, am Besten, wenn EOF auftritt oder 0
                
                
                unsigned char buchstabe = 1;        //decodierter Buchstabe
                
                unsigned int bit_ascii = 0;         //Ascii-Zeichen in dezimal Schreibweise (zw. 0 und 127)
                unsigned int faktor_ascii = 1;      //Faktor der Bitsignifikanz zur Umrechnung binr -> dezimal
                
                unsigned int nr_char = 0;           //counter fuer die Nr des Buchstabens im Ausgangsarray
                unsigned int nr_bit_ascii=0;        //counter fuer die Nr des Bits in den 7bit eines Ascii-Zeichens
                
				meldungsbatchcounter = 0;
                
                
                while((buchstabe!=4)&&(nr_char<299)&&(buchstabe!=0)&&(meldungsbatchcounter<290)){
                    
                    bit_ascii = bit_ascii + ((meldungsbatch[meldungsbatchcounter]) * faktor_ascii);
                    
                    if(nr_bit_ascii>=6){
                        nr_bit_ascii = 0;
                        faktor_ascii = 1;
                        buchstabe = (bit_ascii);
                        if((buchstabe == 4)||(buchstabe == 0))break;
                        meldung[nr_char] = buchstabe;
                        nr_char++;
                        bit_ascii = 0;
                    }else{
                        nr_bit_ascii++;
                        faktor_ascii = faktor_ascii*2;
                    }
                    meldungsbatchcounter++;
                }	
                
                meldung[nr_char+1] = '\0';	//String zur Sicherheit abschlieen
            }
            
            // Funktionnummer in Klartext
            switch(Funktion){
                case 0: strcpy(Funktion_text,"F1");
                    break;
                case 1: strcpy(Funktion_text,"F2");
                    break;
                case 2: strcpy(Funktion_text,"F3");
                    break;
                case 3: strcpy(Funktion_text,"F4");
                    break;
                default:strcpy(Funktion_text, "F?");
                    break;
            }
            
            status = 70;
            
            break;
            
        
            
            
            
// erweiterte CASES
        
        // weitere SCW pruefen (meldung startet nach scw)
        case 50:
            if(bitnr==32){                          // SCW ende, korrekt
                status=51;                          //zwischenstatus wegen erstem bit 
                zeilennr=0;
                bitnr=0;
                break;
            }else if(bitwert!=(SCW[(bitnr-1)])){  // 2. SCW fehlerhaft
                status = 60;                        // Tonruf
                break;
            }
            
            
            
            break;
            
        //Zusatz zu Case 50
        case 51:
            if(bitwert==1){ // 1.bit = 1 => Meldung
                status=6;   // meldung weiter kopieren
            }else{
                status = 60;   // zeile enthaelt keine meldung -> Tonruf
            }
        break;
        
        
        // weitere SCW pruefen (meldung wird nach SCW fortgesetzt
        case 55:
			if(bitnr==32){
				zeilennr=0;
				if(bitwert!=(SCW[(bitnr-1)])){
                    status=7;		//SCW Fehler
                    break;
				}else{				// Ende SCW und korrekt
					zeilennr++;
					status = 6;	
					bitnr = 0;
					wdt_reset();
				}			
			 }else if(bitwert!=(SCW[(bitnr-1)])){
                    status=7;  //SCW Fehler
                    break;
             }
            
        break;   
                
                
            
        // Ohne Meldung (TONRUF)
        case 60:
			wdt_reset();
			wdt_disable();
            signals = 11;
            break;
            
            
        // Daten bereit
        case 70:
			wdt_reset();
			wdt_disable();
            signals = 10;
            break;
            
            
// FEHLERCASES           
            
        // Fehler SCW
        case 90:   
            status = 99;
			wdt_disable();
            wdt_reset();
			signals = 1;
            break;
            
            
        // letzter Case, alle variablen und zahler resetten
        case 99:
            bitnr = 0;
            batchnr=0;
            zeilennr=0;
            parity = 0;
            status = 0;
			wdt_reset();
			wdt_disable();
			signals = 8;
            break;    
            
        default:
            status = 99;
        break;
    }
	
}


// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// Hauptschleife, auf erfolgreiche Decodierung in ISR warten und dann entsp. Funktion aufrufen
// -------------------------------------------------------------------------------------------------------------
int main(void)
{	
	    
	if(MCUSR & (1<<WDRF)){             //wenn Reset durch Watchdog erfolgt ist
        MCUSR &= ~(1<<WDRF);
        wdt_reset();
        wdt_disable();                  //Watchdog ausschalten
		//cli();						muss hier?
		//TIMSK1 &= ~(0b00000010);      muss hier?
		//uart_init();                  muss hier?
		//uart1_init();                 muss hier?
		//sei();                        muss hier?
		_delay_ms(500);
    }else{                              // Reset andere Grnde => Led blinken lassen
		DDRA |= 0b11110000;
		_delay_ms(100);
		PORTA |= 0b11110000;            //leds aus
		PORTA &= 0b01111111;			
		_delay_ms(150);
		PORTA |= 0b10000000;            //leds aus
		PORTA &= 0b10111111;            // leds ein
		_delay_ms(150);
		PORTA |= 0b01000000;            //leds aus
		PORTA &= 0b11011111;            // leds ein
		_delay_ms(150);
		PORTA |= 0b00100000;            //leds aus
		PORTA &= 0b11101111;            // leds ein
		_delay_ms(150);
		PORTA |= 0b00010000;            //leds aus
		PORTA &= 0b11011111;            // leds ein
		_delay_ms(150);
		PORTA |= 0b00100000;            //leds aus
		PORTA &= 0b10111111;            // leds ein
		_delay_ms(150);
		PORTA |= 0b01000000;            //leds aus
		PORTA &= 0b01111111;            // leds ein
		_delay_ms(150);
			
	}
    
	cli();                              //Interrupts deaktivieren
	TIMSK1 &= ~(0b00000010);            // Timer1 Interrupt aus
        
	//Ausgaenge setzen
	DDRA |= 0b11110000;
    //Eingaenge setzen
	DDRB  &= 254;                       //B0 = Eingang vom Empfnger
	PORTB &= 254;                       // PullUp deaktivieren
	
	PORTA |= 0b11110000;                // alle leds aus
	
	uart_init();                        // UART Handy init
	uart1_init();                       // UART PC init
	sei();                              // Interrupts generell aktivieren
	//sms_init();      muss hier?
	
	
//Programm ist in einer Endlosschleife, nach einer dekodierung beginnt es also wieder von vorne    
while(1){
	
		sms_init();						// SMS Funktionen init
		bitnr = 0;                      // Werte auf Startwert stellen
		RIC = 0;
		
		  
    do{
		PORTA |= 0b11110000;            // alle leds aus
		
        RIC_cntr=0;
		
        // Baudrate erkennen und auf Prambelende warten
		int baudratenTimerWert = baudanpassung();               // Baudrate erkennen
		if(baudratenTimerWert>(counterwert+50)){                // Baudrate auf bestimmten Bereich einschrnken
			wait_preamble(counterwert+50);
		}else if(baudratenTimerWert<(counterwert-50)){
			wait_preamble(counterwert-50);
		}else{
			wait_preamble(counterwert);                         // Prambelende abwarten (startet Timer Interrupt selber)
		}
		
        // hier luft Timer und die Decodierung in der ISR
    
        while(signals==0);                      // auf Wertnderung von "signals" warten
        TCCR1B = 0;								//Timer stoppen
        trigger = FALSE;                        // Standardwerte gleich wiederherstellen
        bitnr = 0;
		
		
        switch(signals){                        // Auswerten der Variable "signals". Dadurch signalisiert die ISR, welche Ergebnisart aufgetreten ist
            case 1:                             // signals = 1 => Fehler beim Synchronwort aufgetreten
                uart1_putc('\n');
                uart1_puts("SCW Fehler");       // Debug text ausgeben
                uart1_putc('\n');
                signals = 0;                    // Signals rcksetzen, somit beginnt die Schleife von Neuem
                break;
                
            case 2:                             // signals = 2 => Fehler in der Paritt aufgetreten
                uart1_putc('\n');
                uart1_puts("Parity Fehler");    // Debug text ausgeben
                uart1_putc('\n');
                signals = 0;                    // Signals rcksetzen, somit beginnt die Schleife von Neuem
                break;
				
			case 8:                             // signals = 8 => sonstiger Fehlerfall
				uart1_putc('\n');
                uart1_puts("state99");
				uart1_putc('\n');
                signals = 0;                    // Signals rcksetzen, somit beginnt die Schleife von Neuem
                break;
            
            case 10:                            // signals = 10 => Pagerruf erkannt und mit Meldung erfolgreich dekodiert
				uart1_putc('\n');
                uart1_puts("Pagerruf ");           // Debug text ausgeben
				uart1_puts(Funktion_text);
				uart1_puts(" mit Meldung: ");
				uart1_puts(meldung);
				uart1_putc('\n');
				uart1_putc('\n');
                
                alarmwahl();                    // Funktion zum weiteren Vorgehen (ob SMS gesendet wird und mit welchem Text)
                break;
            
            // Tonruf    
            case 11:                            // signals = 11 => Pagerruf erkannt, jedoch ohne Meldungstext (=Tonruf)
				uart1_putc('\n');
                uart1_puts("Tonruf ");          // Debug text ausgeben
				uart1_puts(Funktion_text);
				uart1_putc('\n');
                
				alarmwahl();                    // Funktion zum weiteren Vorgehen aufrufen
                break;
                
            default:                            // undefinierter Wert fr signals
                uart1_putc('\n');               // Debug text ausgeben
                uart1_puts("unbek. Fehler");
                uart1_putc('\n');
				signals = 0;                    // Signals rcksetzen, somit beginnt die Schleife von Neuem
                break;
        }
        
    }while(signals<10);
    
        // Wenn Pagerruf erkannt, dann hier weitermachen (Fr nchsten Anruf vorbereiten)
        // Werte resetten
		signals = 0;
		wdt_reset();
		
		// meldungsarray leeren
		int i=0;
		meldungsbatchcounter = 0;
		for(i=0;i<300;i++){
			meldungsbatch[i] = 0;
		}
		for(i=0;i<60;i++){
			meldung[i] = 0;
		}
		
		TCCR1B = 0;							// Timer stoppen
		TIMSK1 &= ~(0b00000010);            // Timer1 Interrupt aus
		trigger = FALSE;
		bitnr = 0;
		
		
		wdt_disable();
        _delay_ms(500);
        _delay_ms(500);
		
		PORTA |= 0b11110000;                // alle leds aus
		
	}		
							
	return 0;		
}




// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// Funktion zur Zuweisung eines hinterlegten Hinweistextes zu einem vorliegenden Stichwortkrzels
// -------------------------------------------------------------------------------------------------------------

char *getStichwortText(char * stichwort){
	char klar[15]={0};
	//auswahl
	if((strstr(meldung,"A1"))!=NULL){
		strcpy_P (klar, A1);
	}else if((strstr(meldung,"A2"))!=NULL){
		strcpy_P (klar, A2);
	}else if((strstr(meldung,"A3"))!=NULL){
		strcpy_P (klar, A3);
	}else if((strstr(meldung,"I1"))!=NULL){
		strcpy_P (klar, I1);
	}else if((strstr(meldung,"I2"))!=NULL){
		strcpy_P (klar, I2);
	}else if((strstr(meldung,"I3"))!=NULL){
		strcpy_P (klar, I3);
	}
	
    // Wenn Hinweistext eingefgt, dann noch Linefeed anhngen
	if((strnlen(klar,15))>0){
		strcat(klar,linefeed);
	}
	
	return klar;
	
}



// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// Bei Erkennung einer Nachricht an RIC 1 wird diese Funktion aufgerufen. Darin entscheidung ob SMS gesendet wird
// und Zusammenstellung des SMS-Textes
// -------------------------------------------------------------------------------------------------------------
void RIC1_Alarm (void)
{
	PORTA &= 0b11011111;            // led2 einschalten
	
	char sms_text[150] = {0};       // Variable fr den zu versendenden SMS Text
    int sollsms = 0;                // Variable, ob SMS gesendet werden soll (0=nein, 1=ja)
	
    if((Funktion!=3)&&(Funktion!=2)&&(Funktion!=1)){	//nur wenn F4 weitermachen, sonst abbruch
		return;
	}
    
	PORTA &= 0b10011111;            //led2,3 einschalten
    int i = 0;
    
    while(i<45){                    // Lnge der Meldung ermitteln (max 45 Zeichen)
        if((meldung[i]<33)||(meldung[i]>126)) break;
        i++;
    }
    // i entspricht jetzt der Laenge des Meldungsstrings, ohne nichtdarstellbaren Zeichen
    int length = i;
    
    if(length<7)return; // Wenn weniger als 5 Zeichen Meldung -> return
    
    
    // Auslsen des Stichworts
    char stichwort[8]={0};
	i=0;
	int j = 0;
    while(i<7){
        if(meldung[i]==',')break;
        if(meldung[i]!=' '){
			stichwort[j] = meldung[i];      // Umkopieren
			j++;
		}
        i++;
    }
    
    while(j<7){                         // Rest mit endzeichen fllen
        stichwort[j]='\0';
        j++;
    }
	stichwort[7]='\0';	// Endezeichen auf jedenfall anhngen
    
    // hier hat man das Info-Stichwort (max. 5 Zeichen bis zum Komma) im Char Array 'stichwort'
    // dieses jetzt mit einer Liste vergleichen und wenn Treffer, dann SMS, sonst keine
    
    // mehrere Stichwrter mit gleichem Anfangsbuchstaben
    if( (stichwort[0]=='W') ){
        sollsms=1;
    }
    
    if( ((strcmp(stichwort,"A3"))==0) || ((strcmp(stichwort,"I3"))==0) ){
        sollsms=1;
    }
    
    
    // SMS Alarmieren - Text zusammenbauen und an Handy bergeben
    if(sollsms==1){
        strcat(sms_text,"RIC 1 ");
		strcat(sms_text,getStichwortText(stichwort));
        strncat(sms_text, meldung, 140);
		strcat(sms_text,linefeed);
        strcat(sms_text, "(autom. von Decoder)");
        sms_send("01761234567",sms_text);
        sms_send("01771234567",sms_text);
        
        waitNachAlarm();    // Reload Sperre
        
	}else{  // Debug Ausgabe
		uart1_putc('\n');
		uart1_puts("RIC 1 ohne SMS - ");
		uart1_puts(meldung);
		uart1_putc('\n');
		uart1_putc('\n');
	}
	
}




// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// Funktion zur Erkennung des Prambel-Endes, wird nach Baudratenerkennung aufgerufen
// -------------------------------------------------------------------------------------------------------------
void wait_preamble(int timerwert)			// Timerwert ist das Ergebnis der Baudraten Erkennung
{
    status = 0;
	int bedingung = FALSE;                  // Variable, ob Erkennung abgeschlossen ist
    wdt_enable(WDTO_2S);                    //Watchdog akt., Preambel innerhalb 2s detecten, sonst Reset
    
    do{
        // Timer auf Bitmitte einstellen
		bitnr = 0;
        do{
            //cli();						// Timer1 im CTC Modus mit Overflow Interrupt und Autoreload, Prescaler = 1
            trigger = FALSE;
            TCCR1B =  0b00001000;           // Timer1 anhalten
            OCR1A = (int)(timerwert/2);     // Timer fuer halbes bit vorbereiten
            TIMSK1 |= 0b00000010;           // Timer1 Compare A - Interrupt einstellen
            TCNT1 = 0;
            TCCR1A = 0b00000000;
            while((PINB&1));			// warten solange B0=1;
            while(!(PINB&1));			// warten solange B0=0;
            sei();                          // hier dann B0 = 1
            TCCR1B =  0b00001001;		    // Timer1 mit Prescaler=1 starten
            while(trigger!=TRUE);       // auf Interrupt warten
            TCCR1B =  0b00001000;           // Timer1 anhalten
            trigger = FALSE;
            //cli();
		}	while(!(PINB&1));				//falls nach halber Bitlaenge nicht mehr 1, von vorne
        
        OCR1A = (timerwert);
        TCCR1B =  0b00001001;               // Timer1 mit Prescaler=1 starten
        sei();
        
        
        // Prambelende erkennen
        
        short int bitstrom[2];
        uint8_t wechsel = 0;
        bitnr = 0;
        bitstrom[0]=0;
        bitstrom[1]=0;
        
        while(bedingung!=TRUE){
            
            while(trigger!=TRUE);
            trigger=FALSE;
            
            bitstrom[0] = bitstrom[1];
            bitstrom[1] = bitwert;
            
            if((bitnr<=maxPreamble)&&(wechsel<=minPreamble)&&(bitstrom[0]!=bitstrom[1])){
                wechsel++;
            }else{
                bedingung = FALSE;
            }
            
            if((bitstrom[0]==bitstrom[1])&& ( (wechsel<=minPreamble)||((bitstrom[1]==1)&&(wechsel>minPreamble)))){
                wechsel=0;
            }else{
                bedingung= FALSE;
            }
            
            if((bitnr>maxPreamble)&&( (wechsel<=minPreamble) || ((bitstrom[0]==bitstrom[1])&&(bitstrom[1]==1)) ) ){
                //cli();
				TCCR1B &=  0b11111000;           // Timer1 anhalten
                bitnr=0;
                wechsel=0;
                trigger = FALSE;
                bedingung = FALSE;
                break;
            }else{
                bedingung = FALSE;
            }
            
            if( (bitstrom[0]==bitstrom[1])&&(bitstrom[1]==0)&&(wechsel>minPreamble)){
                bedingung = TRUE;				// Ende Preamble erkannt
                break;
            }else{
                bedingung = FALSE;
            }
        }
    }while(bedingung!=TRUE);		// Solange bis Ende von Preamble erkannt
	wdt_reset();
	bitnr = 1;      // bitnr gleich auf 1 setzen, da erste bit fuer preamble ende erkennung verbraucht wurde
    status = 1;
	sei();
}





// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// Funktion zur genauen Ermittlung der Baudrate
// -------------------------------------------------------------------------------------------------------------
// Rckgabe: Integer der dem Timer1 gegeben werden kann, damit dieser auf die Rate eingestellt ist
// -------------------------------------------------------------------------------------------------------------
int baudanpassung(void){
    
	TIMSK1 &= ~(0b00000010);                            // Timer1 Interrupt aus
	long int baudratentimer = 0;
	long int minBaudTimer = counterwert - 150;          // minWert der Bitlnge
	long int maxBaudTimer = counterwert + 150;          // maxWert der Bitlnge
    
    // dazu Timer starten und stoppen solange B0 = 1, werte aufsummieren wenn im Fenster, dann Mittelwert bilden
	
    do{
        baudratentimer = 0;
        long temp = 0;                                  //Variable um den Mittelwert zu bilden
        int m = 0;                                      //Zhlvariable fr Mittelwertbildung
        
        
        do{                                             //Schleife um 20 mal die Zeit eines High-Bits zu erfassen
            TCNT1 = 0;
            TCCR1A = 0;
            while(( PINB & 1)==0);                      //warten solange B0 = 0;
            
            //mit Prescaler = 1 messen, wie lange 1 anliegt
            TCCR1B = 0b00000001;                        // Prescaler einstellen und Timer starten
            while((( PINB & 1)==1)&&(TCNT1<65535));     //warten solange B0=1
            TCCR1B = 0;                                 //Timer stoppen
            
            if( ( TCNT1 > maxBaudTimer ) || ( TCNT1< minBaudTimer ) ){	//Wenn Zeit nicht passt, alles von vorne
                baudratentimer = 0;
                m=60;
            }else{
                temp += (TCNT1)+1;                      // Zhlerstand hinzufuegen, mit kleinem Offset
                m++;
            }
        }while(m<30);
        
        if(m<55){
            baudratentimer = (temp / m);                // Durchschnitt bilden
        }
        
		
    }while( (baudratentimer<(counterwert-abweichung)) || (baudratentimer>(counterwert+abweichung)) );
	// Wert i.O. => Baudrate erkannt
	
    
	return baudratentimer;
}


// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// Funktion zur Reload Sperre, Zeit etwa 70s => solange LEDs toggeln
// -------------------------------------------------------------------------------------------------------------
void waitNachAlarm(void){
    
	int ledsoll = (PORTA);
    TCCR1B &= 0b11111000;                   // Timer 1 stoppen
	TIMSK1 &= ~(0b00000010);                // Timer1 Interrupt aus
    
    unsigned long int ovfcounter = 0;
	int blinker=0;
    
    TCCR2A &= 0b00000000;
    TCCR2B &= 0b11111000;
    
    TCNT2 = 0;                              // Zhlregister leeren
    TIMSK2 = 0;
	if(TIFR2&1)TIFR2|=1;                    // Bei Bedarf Overflow Flag clear
	
	PORTA |= 0b11110000;                    //leds aus
    
    do{
        TCCR2B |=0b00000111;                // Timer starten mit Prescaler 1024
		while(TCNT2<250);
        TCCR2B &= 0b11111000;               // Timer stoppen
		if(TIFR2&1)TIFR2|=1;                // Bei Bedarf Overflow Flag clear
        TIFR2|=1;                           // OVF Flag clear
        TCNT2 = 0;                          // Counter Register leeren
        ovfcounter++;                       // OVF Counter erhoehen
		
		//LEDs blinken
		blinker++;
		if(blinker>1){
			blinker=0;
			int ledist = (PORTA);
			if((ledist&240)==(ledsoll&240)){
				// leds aus
				PORTA |= 0b11110000;
			}else{
				// leds wieder ein
				PORTA = ledsoll;
			}
		}
    }while(ovfcounter<6500);
    
    
	PORTA |= 0b11110000;			//leds aus
    // Reload Sperre abgeschlossen
}

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// Wickelt die Alarmierung bei erkannter Adresse ab
// -------------------------------------------------------------------------------------------------------------
void alarmwahl(void){
	
    PORTA |= 0b11110000;                                        //leds aus
    unsigned long ric_adresse = RIC;
	char sms_text[150] = {0};
    
    if((Funktion!=1)&&(Funktion!=2)&&(Funktion!=3))return;      // bei F4 und unbekannt, nicht weiterleiten
    
    if(signals==10){    // Alarm mit Meldung
		
        if(ric_adresse==RIC_1){
            RIC1_Alarm();
            
        }else if(ric_adresse==RIC_2){
			PORTA &= 0b10001111;				// led 1,2,3
            strcat(sms_text,"RIC 2 (");
            strcat(sms_text,Funktion_text);
            strcat(sms_text,") ");
			strcat(sms_text,getStichwortText(stichwort));
            strcat(sms_text, meldung);
			strcat(sms_text,linefeed);
            strcat(sms_text, "(autom. von Decoder)");
            sms_send("01761234567",sms_text);
            
            waitNachAlarm();
            
        }else if(ric_adresse==RIC_3){
			PORTA &= 0b00111111;				// led 3,4
            strcat(sms_text,"RIC 3 (");
            strcat(sms_text,Funktion_text);
            strcat(sms_text,") ");
			strcat(sms_text,getStichwortText(stichwort));
            strcat(sms_text, meldung);
			strcat(sms_text,linefeed);
            strcat(sms_text, "(autom. von Decoder)");
            sms_send("01761234567",sms_text);
            sms_send("01771234567",sms_text);
            
            waitNachAlarm();
            
		}else{
            return;
        }
        
        
        
        
    }else{          // Alarm ohne Meldung ( TONRUF )
        
        if(ric_adresse==RIC_1){

            strcat(sms_text,"RIC 1 (");
            strcat(sms_text,Funktion_text);
            strcat(sms_text,") ");
            strcat(sms_text, " - TONRUF - ");
            strcat(sms_text, "        (autom. von Decoder)");
            sms_send("01761234567",sms_text);
            
            waitNachAlarm();
            
        }else if(ric_adresse==RIC_2){
            PORTA &= 0b10001111;				// led 1,2,3
            strcat(sms_text,"RIC 2 (");
            strcat(sms_text,Funktion_text);
            strcat(sms_text,") ");
            strcat(sms_text, " - TONRUF - ");
            strcat(sms_text, "        (autom. von Decoder)");
            sms_send("01761234567",sms_text);
            
            waitNachAlarm();
            
        }else if(ric_adresse==RIC_3){
            PORTA &= 0b00111111;				// led 3,4
            strcat(sms_text,"RIC 3 (");
            strcat(sms_text,Funktion_text);
            strcat(sms_text,") ");
            strcat(sms_text, " - TONRUF - ");
            strcat(sms_text, "        (autom. von Decoder)");
            sms_send("01761234567",sms_text);
            sms_send("01771234567",sms_text);
            
            waitNachAlarm();
            
        }else{
            return;
        }
    }
	
	PORTA |= 0b11110000;			// alle leds aus
    
	uart1_puts(sms_text);           // Debug text ausgeben
    uart1_putc('\n');
    uart1_putc('\n');
}

