// -------------------------------------------------------------------------------------------------------------
// ------------                    UART Routinen fr Atmega324P                        -------------------------
// -------------------------------------------------------------------------------------------------------------
// --------                 November 2012               Autor:  Dane                     -----------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// 2 interne UART.
// Verwendet keine Interrupts !
// -------------------------------------------------------------------------------------------------------------
//                      Quellcode - DATEI
// -------------------------------------------------------------------------------------------------------------


// Include-Files einbinden
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "uart_dual.h"


// -------------------------------------------------------------------------------------------------------------
//                      UART 0
// -------------------------------------------------------------------------------------------------------------
// Initialisierung
void uart_init()
{
	// Einstellen der Baudrate
	uint16_t bubrr1 = (uint16_t) ((uint32_t) F_CPU/(16*BAUDRATE_UART0) - 1);
	UBRR0 = bubrr1;
	
	
	// UART0 Receiver und Transmitter anschalten
	// Data mode 8N1, asynchron 
	UCSR0B = (1 << RXEN0) | (1 << TXEN0);
	UCSR0C = (1 << UCSZ01) | (1 << UCSZ00);
	
	// Dummy-Readout
	do
	{
		// UDR auslesen (Wert wird verworfen)
		UDR0;
	}
	while (UCSR0A & (1 << RXC0));
	

	// Rcksetzen von Receive und Transmit Complete-Flags 
	UCSR0A = (1 << RXC0) | (1 << TXC0);
}



// -------------------------------------------------------------------------------------------------------------
// Senden eines einzelnen Zeichens
void uart_putc (const uint8_t c)
{
	while (!(UCSR0A & (1<<UDRE0)))  ; 			// warten bis Transmit mglich
	UDR0 = c;	
	_delay_us(50);                              // kurz warten
}



// -------------------------------------------------------------------------------------------------------------
// Senden einer Zeichenkette (bis Endezeichen '\0')
void uart_puts (char *s)					
{
	while(*s)
	{
		uart_putc(*s);
		s++;
	}
	 _delay_ms(4);
}


// -------------------------------------------------------------------------------------------------------------
// Empfangen eines einzelnen Char (wartet solange)
unsigned char uart_getc_wait ()
{
	while ( !(UCSR0A & (1<<RXC0)) )
	;
	/* Get and return received data from buffer */
	return UDR0;
}


// -------------------------------------------------------------------------------------------------------------
// Empfangen eines einzelnen Char (wartet nicht, liest aus oder, wenn nicht verfgbar return = -1 )
int uart_getc_nowait ()
{
	if (!(UCSR0A & (1<<RXC0))){   // warten bis Zeichen verfuegbar
			return UDR0;                   // Zeichen aus UDR an Aufrufer zurueckgeben
	}else{
			return -1;
	}	
}

// -------------------------------------------------------------------------------------------------------------
// Puffer leeren
void uart_clear(void){
	int i = 0;
	char a;
	for (i=0;i<20;i++){	
		a = uart_getc_nowait();
	}
}


// -------------------------------------------------------------------------------------------------------------
// Empfangen eines Strings
void uart_gets( char* Buffer, uint8_t MaxLen )
{
  uint8_t NextChar;
  uint8_t StringLen = 0;
 
  NextChar = uart_getc_wait();         // Warte auf und empfange das nchste Zeichen
 
                                  // Sammle solange Zeichen, bis:
                                  // * entweder das String Ende Zeichen kam
                                  // * oder das aufnehmende Array voll ist
  while( NextChar != '\n' && NextChar != '\r' && NextChar != 13 && StringLen < MaxLen - 1 ) {
    *Buffer++ = NextChar;
    StringLen++;
    NextChar = uart_getc_wait();
  }
                            // Noch ein '\0' anhngen um einen Standard
                                  // C-String daraus zu machen
  *Buffer = '\0';
}


// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
//              UART 1
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// Initialisierungsfunktion
void uart1_init()
{
	
	uint16_t bubrr2 = (uint16_t) ((uint32_t) F_CPU/(16*BAUDRATE_UART1) - 1);
	
	//UART1 = PC oder so
	UBRR1H = (uint8_t) (bubrr2>>8);
	UBRR1L = (uint8_t) (bubrr2);
	
	
	// UART1 Receiver und Transmitter anschalten
	// Data mode 8N1, asynchron
	UCSR1B = (1 << RXEN1) | (1 << TXEN1);
	UCSR1C = (1 << UCSZ11) | (1 << UCSZ10);
	
	
	do
	{
		// UDR auslesen (Wert wird verworfen)
		UDR1;
	}
	while (UCSR1A & (1 << RXC1));
	
    
	// Rcksetzen von Receive und Transmit Complete-Flags
	UCSR1A = (1 << RXC1) | (1 << TXC1);
}



// -------------------------------------------------------------------------------------------------------------
//Funktion zum Senden eines einzelnen Zeichens
void uart1_putc (const uint8_t c)
{
	while (!(UCSR1A & (1<<UDRE1)))  ; 			// warten bis Transmit mglich
	UDR1 = c;
	_delay_us(50);
    
}

// Senden einer Zeichenkette (bis Endezeichen '\0')
void uart1_puts (char *s)
{
	while (*s)
	{
		uart1_putc(*s);
		s++;
	}
    _delay_ms(4);                      // nach bertragung kurz warten
}


// -------------------------------------------------------------------------------------------------------------
// Empfangen eines einzelnen Char (solange wird gewartet)
uint8_t uart1_getc_wait ()
{
	while (!(UCSR1A & (1<<RXC1)))   // warten bis Zeichen verfuegbar
        ;
    
    
    return UDR1;                   // Zeichen aus UDR an Aufrufer zurueckgeben
    
}

// -------------------------------------------------------------------------------------------------------------
// Empfangen eines einzelnen Char (wartet nicht, liest Zeichen aus, wenn keines verfgbar return = -1)
int uart1_getc_nowait ()
{
	if (!(UCSR1A & (1<<RXC1))){   // warten bis Zeichen verfuegbar
        return UDR1;                   // Zeichen aus UDR an Aufrufer zurueckgeben
	}else{
        return -1;
	}
}

// -------------------------------------------------------------------------------------------------------------
// Empfangen eines Strings
void uart1_gets( char* Buffer, uint8_t MaxLen )
{
    uint8_t NextChar;
    uint8_t StringLen = 0;
    
    NextChar = uart1_getc_wait();         // Warte auf und empfange das nchste Zeichen
    
    // Sammle solange Zeichen, bis:
    // * entweder das String Ende Zeichen kam
    // * oder das aufnehmende Array voll ist
    while( NextChar != '\n' && NextChar != '\r' && NextChar != 13 && StringLen < MaxLen - 1 ) {
        *Buffer++ = NextChar;
        StringLen++;
        NextChar = uart1_getc_wait();
    }
    
    // Noch ein '\0' anhngen um einen Standard
    // C-String daraus zu machen
    *Buffer = '\0';
}

// -------------------------------------------------------------------------------------------------------------
// Puffer leeren
void uart1_clear(void){
	int i = 0;
	char a;
	for (i=0;i<100;i++){
		a = uart1_getc_nowait();
	}
}


